using NodeCanvas.Framework;
using System;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [GraphInfo(packageName = "NodeCanvas", docsURL = "http://nodecanvas.paradoxnotion.com/documentation/", resourcesURL = "http://nodecanvas.paradoxnotion.com/downloads/", forumsURL = "http://nodecanvas.paradoxnotion.com/forums-page/")]
    public class BehaviourTree : Graph
    {
        [Serializable]
        private struct DerivedSerializationData
        {
            public bool repeat;

            public float updateInterval;
        }

        [SerializeField]
        public bool repeat = true;

        [SerializeField]
        public float updateInterval;

        private float intervalCounter;

        private Status _rootStatus = Status.Resting;

        public Status rootStatus
        {
            get
            {
                return _rootStatus;
            }
            private set
            {
                if (_rootStatus != value)
                {
                    _rootStatus = value;
                    if (onRootStatusChanged != null)
                    {
                        onRootStatusChanged(this, value);
                    }
                }
            }
        }

        public override Type baseNodeType
        {
            get
            {
                return typeof(BTNode);
            }
        }

        public override bool requiresAgent
        {
            get
            {
                return true;
            }
        }

        public override bool requiresPrimeNode
        {
            get
            {
                return true;
            }
        }

        public override bool autoSort
        {
            get
            {
                return true;
            }
        }

        public override bool useLocalBlackboard
        {
            get
            {
                return false;
            }
        }

        public event Action<BehaviourTree, Status> onRootStatusChanged;

        public override object OnDerivedDataSerialization()
        {
            DerivedSerializationData derivedSerializationData = default(DerivedSerializationData);
            derivedSerializationData.repeat = repeat;
            derivedSerializationData.updateInterval = updateInterval;
            return derivedSerializationData;
        }

        public override void OnDerivedDataDeserialization(object data)
        {
            if (data is DerivedSerializationData)
            {
                DerivedSerializationData derivedSerializationData = (DerivedSerializationData)data;
                repeat = derivedSerializationData.repeat;
                DerivedSerializationData derivedSerializationData2 = (DerivedSerializationData)data;
                updateInterval = derivedSerializationData2.updateInterval;
            }
        }

        protected override void OnGraphStarted()
        {
            intervalCounter = updateInterval;
            rootStatus = primeNode.status;
        }

        protected override void OnGraphUpdate()
        {
            if (intervalCounter >= updateInterval)
            {
                intervalCounter = 0f;
                if (Tick(agent, blackboard) != Status.Running && !repeat)
                {
                    Stop(rootStatus == Status.Success);
                }
            }
            if (updateInterval > 0f)
            {
                intervalCounter += Time.deltaTime;
            }
        }

        public Status Tick(Component agent, IBlackboard blackboard)
        {
            if (rootStatus != Status.Running)
            {
                primeNode.Reset();
            }
            rootStatus = primeNode.Execute(agent, blackboard);
            return rootStatus;
        }
    }
}
