using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [Name("Filter")]
    [Category("Decorators")]
    [Description("Filters the access of it's child node either a specific number of times, or every specific amount of time. By default the node is 'Treated as Inactive' to it's parent when child is Filtered. Unchecking this option will instead return Failure when Filtered.")]
    [Icon("Lock", false)]
    public class Filter : BTDecorator
    {
        public enum FilterMode
        {
            LimitNumberOfTimes,
            CoolDown
        }

        public FilterMode filterMode = FilterMode.CoolDown;

        public BBParameter<int> maxCount = new BBParameter<int>
        {
            value = 1
        };

        public BBParameter<float> coolDownTime = new BBParameter<float>
        {
            value = 5f
        };

        public bool inactiveWhenLimited = true;

        private int executedCount;

        private float currentTime;

        public override void OnGraphStarted()
        {
            executedCount = 0;
            currentTime = 0f;
        }

        protected override Status OnExecute(Component agent, IBlackboard blackboard)
        {
            if (decoratedConnection == null)
            {
                return Status.Resting;
            }
            switch (filterMode)
            {
                case FilterMode.CoolDown:
                    if (currentTime > 0f)
                    {
                        return inactiveWhenLimited ? Status.Resting : Status.Failure;
                    }
                    status = decoratedConnection.Execute(agent, blackboard);
                    if (status == Status.Success || status == Status.Failure)
                    {
                        StartCoroutine(Cooldown());
                    }
                    break;
                case FilterMode.LimitNumberOfTimes:
                    if (executedCount >= maxCount.value)
                    {
                        return inactiveWhenLimited ? Status.Resting : Status.Failure;
                    }
                    status = decoratedConnection.Execute(agent, blackboard);
                    if (status == Status.Success || status == Status.Failure)
                    {
                        executedCount++;
                    }
                    break;
            }
            return status;
        }

        private IEnumerator Cooldown()
        {
            for (currentTime = coolDownTime.value; currentTime > 0f; currentTime -= Time.deltaTime)
            {
                yield return null;
            }
        }
    }
}
