using NodeCanvas.Framework;
using NodeCanvas.Framework.Internal;
using ParadoxNotion;
using ParadoxNotion.Design;
using System;
using System.Reflection;
using UnityEngine;

namespace NodeCanvas.Tasks.Conditions
{
    [Description("Check a property on a script and return if it's equal or not to the check value")]
    [Category("✫ Script Control/Standalone Only")]
    public class CheckProperty : ConditionTask, ISubParametersContainer
    {
        [SerializeField]
        protected ReflectedFunctionWrapper functionWrapper;

        [SerializeField]
        protected BBParameter checkValue;

        [SerializeField]
        protected CompareMethod comparison;

        private MethodInfo targetMethod
        {
            get
            {
                return functionWrapper == null ? null : functionWrapper.GetMethod();
            }
        }

        public override Type agentType
        {
            get
            {
                return targetMethod == null ? typeof(Transform) : targetMethod.RTReflectedType();
            }
        }

        protected override string info
        {
            get
            {
                if (functionWrapper == null)
                {
                    return "No Property Selected";
                }
                if (targetMethod == null)
                {
                    return string.Format("<color=#ff6457>* {0} *</color>", functionWrapper.GetMethodString());
                }
                return string.Format("{0}.{1}{2}", agentInfo, targetMethod.Name, OperationTools.GetCompareString(comparison) + checkValue.ToString());
            }
        }

        BBParameter[] ISubParametersContainer.GetIncludeParseParameters()
        {
            return functionWrapper == null ? null : functionWrapper.GetVariables();
        }

        public override void OnValidate(ITaskSystem ownerSystem)
        {
            if (functionWrapper != null && functionWrapper.HasChanged())
            {
                SetMethod(functionWrapper.GetMethod());
            }
            if (functionWrapper != null && targetMethod == null)
            {
                Error(string.Format("Missing Property '{0}'", functionWrapper.GetMethodString()));
            }
        }

        protected override string OnInit()
        {
            if (targetMethod == null)
            {
                return "CheckProperty Error";
            }
            try
            {
                functionWrapper.Init(agent);
                return null;
            }
            catch
            {
                return "CheckProperty Error";
            }
        }

        protected override bool OnCheck()
        {
            if (functionWrapper == null)
            {
                return true;
            }
            if (checkValue.varType == typeof(float))
            {
                return OperationTools.Compare((float)functionWrapper.Call(), (float)checkValue.value, comparison, 0.05f);
            }
            if (checkValue.varType == typeof(int))
            {
                return OperationTools.Compare((int)functionWrapper.Call(), (int)checkValue.value, comparison);
            }
            return Equals(functionWrapper.Call(), checkValue.value);
        }

        private void SetMethod(MethodInfo method)
        {
            if (method != null)
            {
                functionWrapper = ReflectedFunctionWrapper.Create(method, blackboard);
                checkValue = BBParameter.CreateInstance(method.ReturnType, blackboard);
                comparison = CompareMethod.EqualTo;
            }
        }
    }
}
