using System;

namespace MessagePack.LZ4
{
    public static class LZ4Codec
    {
        internal static class HashTablePool
        {
            [ThreadStatic]
            private static ushort[] ushortPool;

            [ThreadStatic]
            private static uint[] uintPool;

            [ThreadStatic]
            private static int[] intPool;

            public static ushort[] GetUShortHashTablePool()
            {
                if (ushortPool == null)
                {
                    ushortPool = new ushort[2048];
                }
                else
                {
                    Array.Clear(ushortPool, 0, ushortPool.Length);
                }
                return ushortPool;
            }

            public static uint[] GetUIntHashTablePool()
            {
                if (uintPool == null)
                {
                    uintPool = new uint[1024];
                }
                else
                {
                    Array.Clear(uintPool, 0, uintPool.Length);
                }
                return uintPool;
            }

            public static int[] GetIntHashTablePool()
            {
                if (intPool == null)
                {
                    intPool = new int[1024];
                }
                else
                {
                    Array.Clear(intPool, 0, intPool.Length);
                }
                return intPool;
            }
        }

        private const int MEMORY_USAGE = 12;

        private const int NOTCOMPRESSIBLE_DETECTIONLEVEL = 6;

        private const int MINMATCH = 4;

        private const int SKIPSTRENGTH = 6;

        private const int COPYLENGTH = 8;

        private const int LASTLITERALS = 5;

        private const int MFLIMIT = 12;

        private const int MINLENGTH = 13;

        private const int MAXD_LOG = 16;

        private const int MAXD = 65536;

        private const int MAXD_MASK = 65535;

        private const int MAX_DISTANCE = 65535;

        private const int ML_BITS = 4;

        private const int ML_MASK = 15;

        private const int RUN_BITS = 4;

        private const int RUN_MASK = 15;

        private const int STEPSIZE_64 = 8;

        private const int STEPSIZE_32 = 4;

        private const int LZ4_64KLIMIT = 65547;

        private const int HASH_LOG = 10;

        private const int HASH_TABLESIZE = 1024;

        private const int HASH_ADJUST = 22;

        private const int HASH64K_LOG = 11;

        private const int HASH64K_TABLESIZE = 2048;

        private const int HASH64K_ADJUST = 21;

        private const int HASHHC_LOG = 15;

        private const int HASHHC_TABLESIZE = 32768;

        private const int HASHHC_ADJUST = 17;

        private static readonly int[] DECODER_TABLE_32 = new int[8]
        {
            0,
            3,
            2,
            3,
            0,
            0,
            0,
            0
        };

        private static readonly int[] DECODER_TABLE_64 = new int[8]
        {
            0,
            0,
            0,
            -1,
            0,
            1,
            2,
            3
        };

        private static readonly int[] DEBRUIJN_TABLE_32 = new int[32]
        {
            0,
            0,
            3,
            0,
            3,
            1,
            3,
            0,
            3,
            2,
            2,
            1,
            3,
            2,
            0,
            1,
            3,
            3,
            1,
            2,
            2,
            2,
            2,
            0,
            3,
            1,
            2,
            0,
            1,
            0,
            1,
            1
        };

        private static readonly int[] DEBRUIJN_TABLE_64 = new int[64]
        {
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            2,
            0,
            3,
            1,
            3,
            1,
            4,
            2,
            7,
            0,
            2,
            3,
            6,
            1,
            5,
            3,
            5,
            1,
            3,
            4,
            4,
            2,
            5,
            6,
            7,
            7,
            0,
            1,
            2,
            3,
            3,
            4,
            6,
            2,
            6,
            5,
            5,
            3,
            4,
            5,
            6,
            7,
            1,
            2,
            4,
            6,
            4,
            4,
            5,
            7,
            2,
            6,
            5,
            7,
            6,
            7,
            7
        };

        private const int MAX_NB_ATTEMPTS = 256;

        private const int OPTIMAL_ML = 18;

        private const int BLOCK_COPY_LIMIT = 16;

        public static int Encode(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            if (IntPtr.Size == 4)
            {
                return Encode32Safe(input, inputOffset, inputLength, output, outputOffset, outputLength);
            }
            return Encode64Safe(input, inputOffset, inputLength, output, outputOffset, outputLength);
        }

        public static int Decode(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            if (IntPtr.Size == 4)
            {
                return Decode32Safe(input, inputOffset, inputLength, output, outputOffset, outputLength);
            }
            return Decode64Safe(input, inputOffset, inputLength, output, outputOffset, outputLength);
        }

        internal static void Poke2(byte[] buffer, int offset, ushort value)
        {
            buffer[offset] = (byte)value;
            buffer[offset + 1] = (byte)(value >> 8);
        }

        internal static ushort Peek2(byte[] buffer, int offset)
        {
            return (ushort)(buffer[offset] | (buffer[offset + 1] << 8));
        }

        internal static uint Peek4(byte[] buffer, int offset)
        {
            return (uint)(buffer[offset] | (buffer[offset + 1] << 8) | (buffer[offset + 2] << 16) | (buffer[offset + 3] << 24));
        }

        private static uint Xor4(byte[] buffer, int offset1, int offset2)
        {
            uint num = (uint)(buffer[offset1] | (buffer[offset1 + 1] << 8) | (buffer[offset1 + 2] << 16) | (buffer[offset1 + 3] << 24));
            uint num2 = (uint)(buffer[offset2] | (buffer[offset2 + 1] << 8) | (buffer[offset2 + 2] << 16) | (buffer[offset2 + 3] << 24));
            return num ^ num2;
        }

        private static ulong Xor8(byte[] buffer, int offset1, int offset2)
        {
            ulong num = buffer[offset1] | ((ulong)buffer[offset1 + 1] << 8) | ((ulong)buffer[offset1 + 2] << 16) | ((ulong)buffer[offset1 + 3] << 24) | ((ulong)buffer[offset1 + 4] << 32) | ((ulong)buffer[offset1 + 5] << 40) | ((ulong)buffer[offset1 + 6] << 48) | ((ulong)buffer[offset1 + 7] << 56);
            ulong num2 = buffer[offset2] | ((ulong)buffer[offset2 + 1] << 8) | ((ulong)buffer[offset2 + 2] << 16) | ((ulong)buffer[offset2 + 3] << 24) | ((ulong)buffer[offset2 + 4] << 32) | ((ulong)buffer[offset2 + 5] << 40) | ((ulong)buffer[offset2 + 6] << 48) | ((ulong)buffer[offset2 + 7] << 56);
            return num ^ num2;
        }

        private static bool Equal2(byte[] buffer, int offset1, int offset2)
        {
            if (buffer[offset1] != buffer[offset2])
            {
                return false;
            }
            return buffer[offset1 + 1] == buffer[offset2 + 1];
        }

        private static bool Equal4(byte[] buffer, int offset1, int offset2)
        {
            if (buffer[offset1] != buffer[offset2])
            {
                return false;
            }
            if (buffer[offset1 + 1] != buffer[offset2 + 1])
            {
                return false;
            }
            if (buffer[offset1 + 2] != buffer[offset2 + 2])
            {
                return false;
            }
            return buffer[offset1 + 3] == buffer[offset2 + 3];
        }

        private static void Copy4(byte[] buf, int src, int dst)
        {
            buf[dst + 3] = buf[src + 3];
            buf[dst + 2] = buf[src + 2];
            buf[dst + 1] = buf[src + 1];
            buf[dst] = buf[src];
        }

        private static void Copy8(byte[] buf, int src, int dst)
        {
            buf[dst + 7] = buf[src + 7];
            buf[dst + 6] = buf[src + 6];
            buf[dst + 5] = buf[src + 5];
            buf[dst + 4] = buf[src + 4];
            buf[dst + 3] = buf[src + 3];
            buf[dst + 2] = buf[src + 2];
            buf[dst + 1] = buf[src + 1];
            buf[dst] = buf[src];
        }

        private static void BlockCopy(byte[] src, int src_0, byte[] dst, int dst_0, int len)
        {
            if (len >= 16)
            {
                Buffer.BlockCopy(src, src_0, dst, dst_0, len);
                return;
            }
            while (len >= 8)
            {
                dst[dst_0] = src[src_0];
                dst[dst_0 + 1] = src[src_0 + 1];
                dst[dst_0 + 2] = src[src_0 + 2];
                dst[dst_0 + 3] = src[src_0 + 3];
                dst[dst_0 + 4] = src[src_0 + 4];
                dst[dst_0 + 5] = src[src_0 + 5];
                dst[dst_0 + 6] = src[src_0 + 6];
                dst[dst_0 + 7] = src[src_0 + 7];
                len -= 8;
                src_0 += 8;
                dst_0 += 8;
            }
            while (len >= 4)
            {
                dst[dst_0] = src[src_0];
                dst[dst_0 + 1] = src[src_0 + 1];
                dst[dst_0 + 2] = src[src_0 + 2];
                dst[dst_0 + 3] = src[src_0 + 3];
                len -= 4;
                src_0 += 4;
                dst_0 += 4;
            }
            while (len-- > 0)
            {
                dst[dst_0++] = src[src_0++];
            }
        }

        private static int WildCopy(byte[] src, int src_0, byte[] dst, int dst_0, int dst_end)
        {
            int num = dst_end - dst_0;
            if (num >= 16)
            {
                Buffer.BlockCopy(src, src_0, dst, dst_0, num);
            }
            else
            {
                while (num >= 4)
                {
                    dst[dst_0] = src[src_0];
                    dst[dst_0 + 1] = src[src_0 + 1];
                    dst[dst_0 + 2] = src[src_0 + 2];
                    dst[dst_0 + 3] = src[src_0 + 3];
                    num -= 4;
                    src_0 += 4;
                    dst_0 += 4;
                }
                while (num-- > 0)
                {
                    dst[dst_0++] = src[src_0++];
                }
            }
            return num;
        }

        private static int SecureCopy(byte[] buffer, int src, int dst, int dst_end)
        {
            int num = dst - src;
            int num2 = dst_end - dst;
            int num3 = num2;
            if (num >= 16)
            {
                if (num >= num2)
                {
                    Buffer.BlockCopy(buffer, src, buffer, dst, num2);
                    return num2;
                }
                do
                {
                    Buffer.BlockCopy(buffer, src, buffer, dst, num);
                    src += num;
                    dst += num;
                    num3 -= num;
                }
                while (num3 >= num);
            }
            while (num3 >= 4)
            {
                buffer[dst] = buffer[src];
                buffer[dst + 1] = buffer[src + 1];
                buffer[dst + 2] = buffer[src + 2];
                buffer[dst + 3] = buffer[src + 3];
                dst += 4;
                src += 4;
                num3 -= 4;
            }
            while (num3-- > 0)
            {
                buffer[dst++] = buffer[src++];
            }
            return num2;
        }

        public static int Encode32Safe(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            CheckArguments(input, inputOffset, inputLength, output, outputOffset, outputLength);
            if (outputLength == 0)
            {
                return 0;
            }
            if (inputLength < 65547)
            {
                ushort[] uShortHashTablePool = HashTablePool.GetUShortHashTablePool();
                return LZ4_compress64kCtx_safe32(uShortHashTablePool, input, output, inputOffset, outputOffset, inputLength, outputLength);
            }
            int[] intHashTablePool = HashTablePool.GetIntHashTablePool();
            return LZ4_compressCtx_safe32(intHashTablePool, input, output, inputOffset, outputOffset, inputLength, outputLength);
        }

        public static int Encode64Safe(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            CheckArguments(input, inputOffset, inputLength, output, outputOffset, outputLength);
            if (outputLength == 0)
            {
                return 0;
            }
            if (inputLength < 65547)
            {
                ushort[] uShortHashTablePool = HashTablePool.GetUShortHashTablePool();
                return LZ4_compress64kCtx_safe64(uShortHashTablePool, input, output, inputOffset, outputOffset, inputLength, outputLength);
            }
            int[] intHashTablePool = HashTablePool.GetIntHashTablePool();
            return LZ4_compressCtx_safe64(intHashTablePool, input, output, inputOffset, outputOffset, inputLength, outputLength);
        }

        public static int Decode32Safe(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            CheckArguments(input, inputOffset, inputLength, output, outputOffset, outputLength);
            if (outputLength == 0)
            {
                return 0;
            }
            int num = LZ4_uncompress_safe32(input, output, inputOffset, outputOffset, outputLength);
            if (num != inputLength)
            {
                throw new ArgumentException("LZ4 block is corrupted, or invalid length has been given.");
            }
            return outputLength;
        }

        public static int Decode64Safe(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            CheckArguments(input, inputOffset, inputLength, output, outputOffset, outputLength);
            if (outputLength == 0)
            {
                return 0;
            }
            int num = LZ4_uncompress_safe64(input, output, inputOffset, outputOffset, outputLength);
            if (num != inputLength)
            {
                throw new ArgumentException("LZ4 block is corrupted, or invalid length has been given.");
            }
            return outputLength;
        }

        private static int LZ4_compressCtx_safe32(int[] hash_table, byte[] src, byte[] dst, int src_0, int dst_0, int src_len, int dst_maxlen)
        {
            int[] dEBRUIJN_TABLE_ = DEBRUIJN_TABLE_32;
            int num = src_0;
            int num2 = src_0 + src_len;
            int num3 = num2 - 12;
            int num4 = dst_0;
            int num5 = num4 + dst_maxlen;
            int num6 = num2 - 5;
            int num7 = num6 - 1;
            int num8 = num6 - 3;
            int num9 = num5 - 6;
            int num10 = num5 - 8;
            if (src_len >= 13)
            {
                hash_table[(uint)((int)Peek4(src, src_0) * -1640531535) >> 22] = src_0 - src_0;
                int num11 = src_0 + 1;
                uint num12 = (uint)((int)Peek4(src, num11) * -1640531535) >> 22;
                while (true)
                {
                    int num13 = 67;
                    int num14 = num11;
                    int num17;
                    while (true)
                    {
                        uint num15 = num12;
                        int num16 = num13++ >> 6;
                        num11 = num14;
                        num14 = num11 + num16;
                        if (num14 > num3)
                        {
                            break;
                        }
                        num12 = (uint)((int)Peek4(src, num14) * -1640531535) >> 22;
                        num17 = src_0 + hash_table[num15];
                        hash_table[num15] = num11 - src_0;
                        if (num17 < num11 - 65535 || !Equal4(src, num17, num11))
                        {
                            continue;
                        }
                        goto IL_00f9;
                    }
                    break;
                IL_03ae:
                    num = num11++;
                    num12 = (uint)((int)Peek4(src, num11) * -1640531535) >> 22;
                    continue;
                IL_00f9:
                    while (num11 > num && num17 > src_0 && src[num11 - 1] == src[num17 - 1])
                    {
                        num11--;
                        num17--;
                    }
                    int num18 = num11 - num;
                    int num19 = num4++;
                    if (num4 + num18 + (num18 >> 8) > num10)
                    {
                        return 0;
                    }
                    if (num18 >= 15)
                    {
                        int num20 = num18 - 15;
                        dst[num19] = 240;
                        if (num20 > 254)
                        {
                            do
                            {
                                dst[num4++] = byte.MaxValue;
                                num20 -= 255;
                            }
                            while (num20 > 254);
                            dst[num4++] = (byte)num20;
                            BlockCopy(src, num, dst, num4, num18);
                            num4 += num18;
                            goto IL_01e1;
                        }
                        dst[num4++] = (byte)num20;
                    }
                    else
                    {
                        dst[num19] = (byte)(num18 << 4);
                    }
                    if (num18 > 0)
                    {
                        int num21 = num4 + num18;
                        WildCopy(src, num, dst, num4, num21);
                        num4 = num21;
                    }
                    goto IL_01e1;
                IL_01e1:
                    while (true)
                    {
                        Poke2(dst, num4, (ushort)(num11 - num17));
                        num4 += 2;
                        num11 += 4;
                        num17 += 4;
                        num = num11;
                        while (true)
                        {
                            if (num11 < num8)
                            {
                                int num22 = (int)Xor4(src, num17, num11);
                                if (num22 == 0)
                                {
                                    num11 += 4;
                                    num17 += 4;
                                    continue;
                                }
                                num11 += dEBRUIJN_TABLE_[(uint)((num22 & -num22) * 125613361) >> 27];
                                break;
                            }
                            if (num11 < num7 && Equal2(src, num17, num11))
                            {
                                num11 += 2;
                                num17 += 2;
                            }
                            if (num11 < num6 && src[num17] == src[num11])
                            {
                                num11++;
                            }
                            break;
                        }
                        num18 = num11 - num;
                        if (num4 + (num18 >> 8) > num9)
                        {
                            return 0;
                        }
                        if (num18 >= 15)
                        {
                            dst[num19] += 15;
                            for (num18 -= 15; num18 > 509; num18 -= 510)
                            {
                                dst[num4++] = byte.MaxValue;
                                dst[num4++] = byte.MaxValue;
                            }
                            if (num18 > 254)
                            {
                                num18 -= 255;
                                dst[num4++] = byte.MaxValue;
                            }
                            dst[num4++] = (byte)num18;
                        }
                        else
                        {
                            dst[num19] += (byte)num18;
                        }
                        if (num11 > num3)
                        {
                            break;
                        }
                        hash_table[(uint)((int)Peek4(src, num11 - 2) * -1640531535) >> 22] = num11 - 2 - src_0;
                        uint num15 = (uint)((int)Peek4(src, num11) * -1640531535) >> 22;
                        num17 = src_0 + hash_table[num15];
                        hash_table[num15] = num11 - src_0;
                        if (num17 > num11 - 65536 && Equal4(src, num17, num11))
                        {
                            num19 = num4++;
                            dst[num19] = 0;
                            continue;
                        }
                        goto IL_03ae;
                    }
                    num = num11;
                    break;
                }
            }
            int num23 = num2 - num;
            if (num4 + num23 + 1 + (num23 + 255 - 15) / 255 > num5)
            {
                return 0;
            }
            if (num23 >= 15)
            {
                dst[num4++] = 240;
                for (num23 -= 15; num23 > 254; num23 -= 255)
                {
                    dst[num4++] = byte.MaxValue;
                }
                dst[num4++] = (byte)num23;
            }
            else
            {
                dst[num4++] = (byte)(num23 << 4);
            }
            BlockCopy(src, num, dst, num4, num2 - num);
            num4 += num2 - num;
            return num4 - dst_0;
        }

        private static int LZ4_compress64kCtx_safe32(ushort[] hash_table, byte[] src, byte[] dst, int src_0, int dst_0, int src_len, int dst_maxlen)
        {
            int[] dEBRUIJN_TABLE_ = DEBRUIJN_TABLE_32;
            int num = src_0;
            int num2 = src_0 + src_len;
            int num3 = num2 - 12;
            int num4 = dst_0;
            int num5 = num4 + dst_maxlen;
            int num6 = num2 - 5;
            int num7 = num6 - 1;
            int num8 = num6 - 3;
            int num9 = num5 - 6;
            int num10 = num5 - 8;
            if (src_len >= 13)
            {
                int num11 = src_0 + 1;
                uint num12 = (uint)((int)Peek4(src, num11) * -1640531535) >> 21;
                while (true)
                {
                    int num13 = 67;
                    int num14 = num11;
                    int num17;
                    while (true)
                    {
                        uint num15 = num12;
                        int num16 = num13++ >> 6;
                        num11 = num14;
                        num14 = num11 + num16;
                        if (num14 > num3)
                        {
                            break;
                        }
                        num12 = (uint)((int)Peek4(src, num14) * -1640531535) >> 21;
                        num17 = src_0 + hash_table[num15];
                        hash_table[num15] = (ushort)(num11 - src_0);
                        if (!Equal4(src, num17, num11))
                        {
                            continue;
                        }
                        goto IL_00d8;
                    }
                    break;
                IL_037d:
                    num = num11++;
                    num12 = (uint)((int)Peek4(src, num11) * -1640531535) >> 21;
                    continue;
                IL_00d8:
                    while (num11 > num && num17 > src_0 && src[num11 - 1] == src[num17 - 1])
                    {
                        num11--;
                        num17--;
                    }
                    int num18 = num11 - num;
                    int num19 = num4++;
                    if (num4 + num18 + (num18 >> 8) > num10)
                    {
                        return 0;
                    }
                    if (num18 >= 15)
                    {
                        int num20 = num18 - 15;
                        dst[num19] = 240;
                        if (num20 > 254)
                        {
                            do
                            {
                                dst[num4++] = byte.MaxValue;
                                num20 -= 255;
                            }
                            while (num20 > 254);
                            dst[num4++] = (byte)num20;
                            BlockCopy(src, num, dst, num4, num18);
                            num4 += num18;
                            goto IL_01bc;
                        }
                        dst[num4++] = (byte)num20;
                    }
                    else
                    {
                        dst[num19] = (byte)(num18 << 4);
                    }
                    if (num18 > 0)
                    {
                        int num21 = num4 + num18;
                        WildCopy(src, num, dst, num4, num21);
                        num4 = num21;
                    }
                    goto IL_01bc;
                IL_01bc:
                    while (true)
                    {
                        Poke2(dst, num4, (ushort)(num11 - num17));
                        num4 += 2;
                        num11 += 4;
                        num17 += 4;
                        num = num11;
                        while (true)
                        {
                            if (num11 < num8)
                            {
                                int num22 = (int)Xor4(src, num17, num11);
                                if (num22 == 0)
                                {
                                    num11 += 4;
                                    num17 += 4;
                                    continue;
                                }
                                num11 += dEBRUIJN_TABLE_[(uint)((num22 & -num22) * 125613361) >> 27];
                                break;
                            }
                            if (num11 < num7 && Equal2(src, num17, num11))
                            {
                                num11 += 2;
                                num17 += 2;
                            }
                            if (num11 < num6 && src[num17] == src[num11])
                            {
                                num11++;
                            }
                            break;
                        }
                        int num20 = num11 - num;
                        if (num4 + (num20 >> 8) > num9)
                        {
                            return 0;
                        }
                        if (num20 >= 15)
                        {
                            dst[num19] += 15;
                            for (num20 -= 15; num20 > 509; num20 -= 510)
                            {
                                dst[num4++] = byte.MaxValue;
                                dst[num4++] = byte.MaxValue;
                            }
                            if (num20 > 254)
                            {
                                num20 -= 255;
                                dst[num4++] = byte.MaxValue;
                            }
                            dst[num4++] = (byte)num20;
                        }
                        else
                        {
                            dst[num19] += (byte)num20;
                        }
                        if (num11 > num3)
                        {
                            break;
                        }
                        hash_table[(uint)((int)Peek4(src, num11 - 2) * -1640531535) >> 21] = (ushort)(num11 - 2 - src_0);
                        uint num15 = (uint)((int)Peek4(src, num11) * -1640531535) >> 21;
                        num17 = src_0 + hash_table[num15];
                        hash_table[num15] = (ushort)(num11 - src_0);
                        if (Equal4(src, num17, num11))
                        {
                            num19 = num4++;
                            dst[num19] = 0;
                            continue;
                        }
                        goto IL_037d;
                    }
                    num = num11;
                    break;
                }
            }
            int num23 = num2 - num;
            if (num4 + num23 + 1 + (num23 - 15 + 255) / 255 > num5)
            {
                return 0;
            }
            if (num23 >= 15)
            {
                dst[num4++] = 240;
                for (num23 -= 15; num23 > 254; num23 -= 255)
                {
                    dst[num4++] = byte.MaxValue;
                }
                dst[num4++] = (byte)num23;
            }
            else
            {
                dst[num4++] = (byte)(num23 << 4);
            }
            BlockCopy(src, num, dst, num4, num2 - num);
            num4 += num2 - num;
            return num4 - dst_0;
        }

        private static int LZ4_uncompress_safe32(byte[] src, byte[] dst, int src_0, int dst_0, int dst_len)
        {
            int[] dECODER_TABLE_ = DECODER_TABLE_32;
            int num = src_0;
            int num2 = dst_0;
            int num3 = num2 + dst_len;
            int num4 = num3 - 5;
            int num5 = num3 - 8;
            int num6 = num3 - 8;
            while (true)
            {
                byte b = src[num++];
                int num7;
                if ((num7 = b >> 4) == 15)
                {
                    int num8;
                    while ((num8 = src[num++]) == 255)
                    {
                        num7 += 255;
                    }
                    num7 += num8;
                }
                int num9 = num2 + num7;
                if (num9 > num5)
                {
                    if (num9 != num3)
                    {
                        break;
                    }
                    BlockCopy(src, num, dst, num2, num7);
                    num += num7;
                    return num - src_0;
                }
                if (num2 < num9)
                {
                    int num10 = WildCopy(src, num, dst, num2, num9);
                    num += num10;
                    num2 += num10;
                }
                num -= num2 - num9;
                num2 = num9;
                int num11 = num9 - Peek2(src, num);
                num += 2;
                if (num11 < dst_0)
                {
                    break;
                }
                if ((num7 = b & 0xF) == 15)
                {
                    while (src[num] == byte.MaxValue)
                    {
                        num++;
                        num7 += 255;
                    }
                    num7 += src[num++];
                }
                if (num2 - num11 < 4)
                {
                    dst[num2] = dst[num11];
                    dst[num2 + 1] = dst[num11 + 1];
                    dst[num2 + 2] = dst[num11 + 2];
                    dst[num2 + 3] = dst[num11 + 3];
                    num2 += 4;
                    num11 += 4;
                    num11 -= dECODER_TABLE_[num2 - num11];
                    Copy4(dst, num11, num2);
                }
                else
                {
                    Copy4(dst, num11, num2);
                    num2 += 4;
                    num11 += 4;
                }
                num9 = num2 + num7;
                if (num9 > num6)
                {
                    if (num9 > num4)
                    {
                        break;
                    }
                    if (num2 < num5)
                    {
                        int num10 = SecureCopy(dst, num11, num2, num5);
                        num11 += num10;
                        num2 += num10;
                    }
                    while (num2 < num9)
                    {
                        dst[num2++] = dst[num11++];
                    }
                    num2 = num9;
                }
                else
                {
                    if (num2 < num9)
                    {
                        SecureCopy(dst, num11, num2, num9);
                    }
                    num2 = num9;
                }
            }
            return -(num - src_0);
        }

        private static int LZ4_compressCtx_safe64(int[] hash_table, byte[] src, byte[] dst, int src_0, int dst_0, int src_len, int dst_maxlen)
        {
            int[] dEBRUIJN_TABLE_ = DEBRUIJN_TABLE_64;
            int num = src_0;
            int num2 = src_0 + src_len;
            int num3 = num2 - 12;
            int num4 = dst_0;
            int num5 = num4 + dst_maxlen;
            int num6 = num2 - 5;
            int num7 = num6 - 1;
            int num8 = num6 - 3;
            int num9 = num6 - 7;
            int num10 = num5 - 6;
            int num11 = num5 - 8;
            if (src_len >= 13)
            {
                hash_table[(uint)((int)Peek4(src, src_0) * -1640531535) >> 22] = src_0 - src_0;
                int num12 = src_0 + 1;
                uint num13 = (uint)((int)Peek4(src, num12) * -1640531535) >> 22;
                while (true)
                {
                    int num14 = 67;
                    int num15 = num12;
                    int num18;
                    while (true)
                    {
                        uint num16 = num13;
                        int num17 = num14++ >> 6;
                        num12 = num15;
                        num15 = num12 + num17;
                        if (num15 > num3)
                        {
                            break;
                        }
                        num13 = (uint)((int)Peek4(src, num15) * -1640531535) >> 22;
                        num18 = src_0 + hash_table[num16];
                        hash_table[num16] = num12 - src_0;
                        if (num18 < num12 - 65535 || !Equal4(src, num18, num12))
                        {
                            continue;
                        }
                        goto IL_00ff;
                    }
                    break;
                IL_03da:
                    num = num12++;
                    num13 = (uint)((int)Peek4(src, num12) * -1640531535) >> 22;
                    continue;
                IL_00ff:
                    while (num12 > num && num18 > src_0 && src[num12 - 1] == src[num18 - 1])
                    {
                        num12--;
                        num18--;
                    }
                    int num19 = num12 - num;
                    int num20 = num4++;
                    if (num4 + num19 + (num19 >> 8) > num11)
                    {
                        return 0;
                    }
                    if (num19 >= 15)
                    {
                        int num21 = num19 - 15;
                        dst[num20] = 240;
                        if (num21 > 254)
                        {
                            do
                            {
                                dst[num4++] = byte.MaxValue;
                                num21 -= 255;
                            }
                            while (num21 > 254);
                            dst[num4++] = (byte)num21;
                            BlockCopy(src, num, dst, num4, num19);
                            num4 += num19;
                            goto IL_01e7;
                        }
                        dst[num4++] = (byte)num21;
                    }
                    else
                    {
                        dst[num20] = (byte)(num19 << 4);
                    }
                    if (num19 > 0)
                    {
                        int num22 = num4 + num19;
                        WildCopy(src, num, dst, num4, num22);
                        num4 = num22;
                    }
                    goto IL_01e7;
                IL_01e7:
                    while (true)
                    {
                        Poke2(dst, num4, (ushort)(num12 - num18));
                        num4 += 2;
                        num12 += 4;
                        num18 += 4;
                        num = num12;
                        while (true)
                        {
                            if (num12 < num9)
                            {
                                long num23 = (long)Xor8(src, num18, num12);
                                if (num23 == 0)
                                {
                                    num12 += 8;
                                    num18 += 8;
                                    continue;
                                }
                                num12 += dEBRUIJN_TABLE_[(ulong)((num23 & -num23) * 151050438428048703L) >> 58];
                                break;
                            }
                            if (num12 < num8 && Equal4(src, num18, num12))
                            {
                                num12 += 4;
                                num18 += 4;
                            }
                            if (num12 < num7 && Equal2(src, num18, num12))
                            {
                                num12 += 2;
                                num18 += 2;
                            }
                            if (num12 < num6 && src[num18] == src[num12])
                            {
                                num12++;
                            }
                            break;
                        }
                        num19 = num12 - num;
                        if (num4 + (num19 >> 8) > num10)
                        {
                            return 0;
                        }
                        if (num19 >= 15)
                        {
                            dst[num20] += 15;
                            for (num19 -= 15; num19 > 509; num19 -= 510)
                            {
                                dst[num4++] = byte.MaxValue;
                                dst[num4++] = byte.MaxValue;
                            }
                            if (num19 > 254)
                            {
                                num19 -= 255;
                                dst[num4++] = byte.MaxValue;
                            }
                            dst[num4++] = (byte)num19;
                        }
                        else
                        {
                            dst[num20] += (byte)num19;
                        }
                        if (num12 > num3)
                        {
                            break;
                        }
                        hash_table[(uint)((int)Peek4(src, num12 - 2) * -1640531535) >> 22] = num12 - 2 - src_0;
                        uint num16 = (uint)((int)Peek4(src, num12) * -1640531535) >> 22;
                        num18 = src_0 + hash_table[num16];
                        hash_table[num16] = num12 - src_0;
                        if (num18 > num12 - 65536 && Equal4(src, num18, num12))
                        {
                            num20 = num4++;
                            dst[num20] = 0;
                            continue;
                        }
                        goto IL_03da;
                    }
                    num = num12;
                    break;
                }
            }
            int num24 = num2 - num;
            if (num4 + num24 + 1 + (num24 + 255 - 15) / 255 > num5)
            {
                return 0;
            }
            if (num24 >= 15)
            {
                dst[num4++] = 240;
                for (num24 -= 15; num24 > 254; num24 -= 255)
                {
                    dst[num4++] = byte.MaxValue;
                }
                dst[num4++] = (byte)num24;
            }
            else
            {
                dst[num4++] = (byte)(num24 << 4);
            }
            BlockCopy(src, num, dst, num4, num2 - num);
            num4 += num2 - num;
            return num4 - dst_0;
        }

        private static int LZ4_compress64kCtx_safe64(ushort[] hash_table, byte[] src, byte[] dst, int src_0, int dst_0, int src_len, int dst_maxlen)
        {
            int[] dEBRUIJN_TABLE_ = DEBRUIJN_TABLE_64;
            int num = src_0;
            int num2 = src_0 + src_len;
            int num3 = num2 - 12;
            int num4 = dst_0;
            int num5 = num4 + dst_maxlen;
            int num6 = num2 - 5;
            int num7 = num6 - 1;
            int num8 = num6 - 3;
            int num9 = num6 - 7;
            int num10 = num5 - 6;
            int num11 = num5 - 8;
            if (src_len >= 13)
            {
                int num12 = src_0 + 1;
                uint num13 = (uint)((int)Peek4(src, num12) * -1640531535) >> 21;
                while (true)
                {
                    int num14 = 67;
                    int num15 = num12;
                    int num18;
                    while (true)
                    {
                        uint num16 = num13;
                        int num17 = num14++ >> 6;
                        num12 = num15;
                        num15 = num12 + num17;
                        if (num15 > num3)
                        {
                            break;
                        }
                        num13 = (uint)((int)Peek4(src, num15) * -1640531535) >> 21;
                        num18 = src_0 + hash_table[num16];
                        hash_table[num16] = (ushort)(num12 - src_0);
                        if (!Equal4(src, num18, num12))
                        {
                            continue;
                        }
                        goto IL_00de;
                    }
                    break;
                IL_03a9:
                    num = num12++;
                    num13 = (uint)((int)Peek4(src, num12) * -1640531535) >> 21;
                    continue;
                IL_00de:
                    while (num12 > num && num18 > src_0 && src[num12 - 1] == src[num18 - 1])
                    {
                        num12--;
                        num18--;
                    }
                    int num19 = num12 - num;
                    int num20 = num4++;
                    if (num4 + num19 + (num19 >> 8) > num11)
                    {
                        return 0;
                    }
                    if (num19 >= 15)
                    {
                        int num21 = num19 - 15;
                        dst[num20] = 240;
                        if (num21 > 254)
                        {
                            do
                            {
                                dst[num4++] = byte.MaxValue;
                                num21 -= 255;
                            }
                            while (num21 > 254);
                            dst[num4++] = (byte)num21;
                            BlockCopy(src, num, dst, num4, num19);
                            num4 += num19;
                            goto IL_01c2;
                        }
                        dst[num4++] = (byte)num21;
                    }
                    else
                    {
                        dst[num20] = (byte)(num19 << 4);
                    }
                    if (num19 > 0)
                    {
                        int num22 = num4 + num19;
                        WildCopy(src, num, dst, num4, num22);
                        num4 = num22;
                    }
                    goto IL_01c2;
                IL_01c2:
                    while (true)
                    {
                        Poke2(dst, num4, (ushort)(num12 - num18));
                        num4 += 2;
                        num12 += 4;
                        num18 += 4;
                        num = num12;
                        while (true)
                        {
                            if (num12 < num9)
                            {
                                long num23 = (long)Xor8(src, num18, num12);
                                if (num23 == 0)
                                {
                                    num12 += 8;
                                    num18 += 8;
                                    continue;
                                }
                                num12 += dEBRUIJN_TABLE_[(ulong)((num23 & -num23) * 151050438428048703L) >> 58];
                                break;
                            }
                            if (num12 < num8 && Equal4(src, num18, num12))
                            {
                                num12 += 4;
                                num18 += 4;
                            }
                            if (num12 < num7 && Equal2(src, num18, num12))
                            {
                                num12 += 2;
                                num18 += 2;
                            }
                            if (num12 < num6 && src[num18] == src[num12])
                            {
                                num12++;
                            }
                            break;
                        }
                        int num21 = num12 - num;
                        if (num4 + (num21 >> 8) > num10)
                        {
                            return 0;
                        }
                        if (num21 >= 15)
                        {
                            dst[num20] += 15;
                            for (num21 -= 15; num21 > 509; num21 -= 510)
                            {
                                dst[num4++] = byte.MaxValue;
                                dst[num4++] = byte.MaxValue;
                            }
                            if (num21 > 254)
                            {
                                num21 -= 255;
                                dst[num4++] = byte.MaxValue;
                            }
                            dst[num4++] = (byte)num21;
                        }
                        else
                        {
                            dst[num20] += (byte)num21;
                        }
                        if (num12 > num3)
                        {
                            break;
                        }
                        hash_table[(uint)((int)Peek4(src, num12 - 2) * -1640531535) >> 21] = (ushort)(num12 - 2 - src_0);
                        uint num16 = (uint)((int)Peek4(src, num12) * -1640531535) >> 21;
                        num18 = src_0 + hash_table[num16];
                        hash_table[num16] = (ushort)(num12 - src_0);
                        if (Equal4(src, num18, num12))
                        {
                            num20 = num4++;
                            dst[num20] = 0;
                            continue;
                        }
                        goto IL_03a9;
                    }
                    num = num12;
                    break;
                }
            }
            int num24 = num2 - num;
            if (num4 + num24 + 1 + (num24 - 15 + 255) / 255 > num5)
            {
                return 0;
            }
            if (num24 >= 15)
            {
                dst[num4++] = 240;
                for (num24 -= 15; num24 > 254; num24 -= 255)
                {
                    dst[num4++] = byte.MaxValue;
                }
                dst[num4++] = (byte)num24;
            }
            else
            {
                dst[num4++] = (byte)(num24 << 4);
            }
            BlockCopy(src, num, dst, num4, num2 - num);
            num4 += num2 - num;
            return num4 - dst_0;
        }

        private static int LZ4_uncompress_safe64(byte[] src, byte[] dst, int src_0, int dst_0, int dst_len)
        {
            int[] dECODER_TABLE_ = DECODER_TABLE_32;
            int[] dECODER_TABLE_2 = DECODER_TABLE_64;
            int num = src_0;
            int num2 = dst_0;
            int num3 = num2 + dst_len;
            int num4 = num3 - 5;
            int num5 = num3 - 8;
            int num6 = num3 - 8 - 4;
            while (true)
            {
                uint num7 = src[num++];
                int num8;
                if ((num8 = (byte)(num7 >> 4)) == 15)
                {
                    int num9;
                    while ((num9 = src[num++]) == 255)
                    {
                        num8 += 255;
                    }
                    num8 += num9;
                }
                int num10 = num2 + num8;
                if (num10 > num5)
                {
                    if (num10 != num3)
                    {
                        break;
                    }
                    BlockCopy(src, num, dst, num2, num8);
                    num += num8;
                    return num - src_0;
                }
                if (num2 < num10)
                {
                    int num11 = WildCopy(src, num, dst, num2, num10);
                    num += num11;
                    num2 += num11;
                }
                num -= num2 - num10;
                num2 = num10;
                int num12 = num10 - Peek2(src, num);
                num += 2;
                if (num12 < dst_0)
                {
                    break;
                }
                if ((num8 = (byte)(num7 & 0xF)) == 15)
                {
                    while (src[num] == byte.MaxValue)
                    {
                        num++;
                        num8 += 255;
                    }
                    num8 += src[num++];
                }
                if (num2 - num12 < 8)
                {
                    int num13 = dECODER_TABLE_2[num2 - num12];
                    dst[num2] = dst[num12];
                    dst[num2 + 1] = dst[num12 + 1];
                    dst[num2 + 2] = dst[num12 + 2];
                    dst[num2 + 3] = dst[num12 + 3];
                    num2 += 4;
                    num12 += 4;
                    num12 -= dECODER_TABLE_[num2 - num12];
                    Copy4(dst, num12, num2);
                    num2 += 4;
                    num12 -= num13;
                }
                else
                {
                    Copy8(dst, num12, num2);
                    num2 += 8;
                    num12 += 8;
                }
                num10 = num2 + num8 - 4;
                if (num10 > num6)
                {
                    if (num10 > num4)
                    {
                        break;
                    }
                    if (num2 < num5)
                    {
                        int num11 = SecureCopy(dst, num12, num2, num5);
                        num12 += num11;
                        num2 += num11;
                    }
                    while (num2 < num10)
                    {
                        dst[num2++] = dst[num12++];
                    }
                    num2 = num10;
                }
                else
                {
                    if (num2 < num10)
                    {
                        SecureCopy(dst, num12, num2, num10);
                    }
                    num2 = num10;
                }
            }
            return -(num - src_0);
        }

        public static int MaximumOutputLength(int inputLength)
        {
            return inputLength + inputLength / 255 + 16;
        }

        internal static void CheckArguments(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset, int outputLength)
        {
            if (inputLength == 0)
            {
                return;
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            if ((uint)inputOffset > (uint)input.Length)
            {
                throw new ArgumentOutOfRangeException("inputOffset");
            }
            if ((uint)inputLength > (uint)(input.Length - inputOffset))
            {
                throw new ArgumentOutOfRangeException("inputLength");
            }
            if (output == null)
            {
                throw new ArgumentNullException("output");
            }
            if ((uint)outputOffset > (uint)output.Length)
            {
                throw new ArgumentOutOfRangeException("outputOffset");
            }
            if ((uint)outputLength <= (uint)(output.Length - outputOffset))
            {
                return;
            }
            throw new ArgumentOutOfRangeException("outputLength");
        }
    }
}
