using System;

namespace UniRx.Operators
{
    internal class StartObservable<T> : OperatorObservableBase<T>
    {
        private class StartObserver : OperatorObserverBase<T, T>
        {
            private readonly StartObservable<T> parent;

            public StartObserver(StartObservable<T> parent, IObserver<T> observer, IDisposable cancel)
                : base(observer, cancel)
            {
                this.parent = parent;
            }

            public void Run()
            {
                T value = default(T);
                try
                {
                    if (parent.function != null)
                    {
                        value = parent.function();
                    }
                    else
                    {
                        parent.action();
                    }
                }
                catch (Exception error)
                {
                    try
                    {
                        observer.OnError(error);
                    }
                    finally
                    {
                        Dispose();
                    }
                    return;
                }
                OnNext(value);
                try
                {
                    observer.OnCompleted();
                }
                finally
                {
                    Dispose();
                }
            }

            public override void OnNext(T value)
            {
                try
                {
                    observer.OnNext(value);
                }
                catch
                {
                    Dispose();
                    throw;
                }
            }

            public override void OnError(Exception error)
            {
                try
                {
                    observer.OnError(error);
                }
                finally
                {
                    Dispose();
                }
            }

            public override void OnCompleted()
            {
                try
                {
                    observer.OnCompleted();
                }
                finally
                {
                    Dispose();
                }
            }
        }

        private readonly Action action;

        private readonly Func<T> function;

        private readonly IScheduler scheduler;

        private readonly TimeSpan? startAfter;

        public StartObservable(Func<T> function, TimeSpan? startAfter, IScheduler scheduler)
            : base(scheduler == Scheduler.CurrentThread)
        {
            this.function = function;
            this.startAfter = startAfter;
            this.scheduler = scheduler;
        }

        public StartObservable(Action action, TimeSpan? startAfter, IScheduler scheduler)
            : base(scheduler == Scheduler.CurrentThread)
        {
            this.action = action;
            this.startAfter = startAfter;
            this.scheduler = scheduler;
        }

        protected override IDisposable SubscribeCore(IObserver<T> observer, IDisposable cancel)
        {
            if (startAfter.HasValue)
            {
                return scheduler.Schedule(startAfter.Value, new StartObserver(this, observer, cancel).Run);
            }
            return scheduler.Schedule(new StartObserver(this, observer, cancel).Run);
        }
    }
}
