using System;
using System.Collections.Generic;
using System.Threading;
using UnityEngine;

namespace UniRx
{
    [Serializable]
    public class ReactiveProperty<T> : IReactiveProperty<T>, IDisposable, IOptimizedObservable<T>, IReadOnlyReactiveProperty<T>, IObservable<T>
    {
        private class ReactivePropertyObserver : IObserver<T>
        {
            private readonly ReactiveProperty<T> parent;

            private int isStopped;

            public ReactivePropertyObserver(ReactiveProperty<T> parent)
            {
                this.parent = parent;
            }

            public void OnNext(T value)
            {
                parent.Value = value;
            }

            public void OnError(Exception error)
            {
                if (Interlocked.Increment(ref isStopped) == 1)
                {
                    parent.lastException = error;
                    Subject<T> publisher = parent.publisher;
                    if (publisher != null)
                    {
                        publisher.OnError(error);
                    }
                    parent.Dispose();
                }
            }

            public void OnCompleted()
            {
                if (Interlocked.Increment(ref isStopped) == 1)
                {
                    IDisposable sourceConnection = parent.sourceConnection;
                    parent.sourceConnection = null;
                    if (sourceConnection != null)
                    {
                        sourceConnection.Dispose();
                    }
                }
            }
        }

        private static readonly IEqualityComparer<T> defaultEqualityComparer = UnityEqualityComparer.GetDefault<T>();

        [NonSerialized]
        private bool canPublishValueOnSubscribe;

        [NonSerialized]
        private bool isDisposed;

        [SerializeField]
        private T value = default(T);

        [NonSerialized]
        private Subject<T> publisher;

        [NonSerialized]
        private IDisposable sourceConnection;

        [NonSerialized]
        private Exception lastException;

        protected virtual IEqualityComparer<T> EqualityComparer
        {
            get
            {
                return defaultEqualityComparer;
            }
        }

        public T Value
        {
            get
            {
                return value;
            }
            set
            {
                if (!canPublishValueOnSubscribe)
                {
                    canPublishValueOnSubscribe = true;
                    SetValue(value);
                    if (!isDisposed)
                    {
                        Subject<T> subject = publisher;
                        if (subject != null)
                        {
                            subject.OnNext(this.value);
                        }
                    }
                }
                else
                {
                    if (EqualityComparer.Equals(this.value, value))
                    {
                        return;
                    }
                    SetValue(value);
                    if (!isDisposed)
                    {
                        Subject<T> subject2 = publisher;
                        if (subject2 != null)
                        {
                            subject2.OnNext(this.value);
                        }
                    }
                }
            }
        }

        public bool HasValue
        {
            get
            {
                return canPublishValueOnSubscribe;
            }
        }

        public ReactiveProperty()
            : this(default(T))
        {
        }

        public ReactiveProperty(T initialValue)
        {
            SetValue(initialValue);
            canPublishValueOnSubscribe = true;
        }

        public ReactiveProperty(IObservable<T> source)
        {
            canPublishValueOnSubscribe = false;
            sourceConnection = source.Subscribe(new ReactivePropertyObserver(this));
        }

        public ReactiveProperty(IObservable<T> source, T initialValue)
        {
            canPublishValueOnSubscribe = false;
            Value = initialValue;
            sourceConnection = source.Subscribe(new ReactivePropertyObserver(this));
        }

        protected virtual void SetValue(T value)
        {
            this.value = value;
        }

        public void SetValueAndForceNotify(T value)
        {
            SetValue(value);
            if (!isDisposed)
            {
                Subject<T> subject = publisher;
                if (subject != null)
                {
                    subject.OnNext(this.value);
                }
            }
        }

        public IDisposable Subscribe(IObserver<T> observer)
        {
            if (lastException != null)
            {
                observer.OnError(lastException);
                return Disposable.Empty;
            }
            if (isDisposed)
            {
                observer.OnCompleted();
                return Disposable.Empty;
            }
            if (publisher == null)
            {
                publisher = new Subject<T>();
            }
            Subject<T> subject = publisher;
            if (subject != null)
            {
                IDisposable result = subject.Subscribe(observer);
                if (canPublishValueOnSubscribe)
                {
                    observer.OnNext(value);
                }
                return result;
            }
            observer.OnCompleted();
            return Disposable.Empty;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!isDisposed)
            {
                isDisposed = true;
                IDisposable disposable = sourceConnection;
                if (disposable != null)
                {
                    disposable.Dispose();
                    sourceConnection = null;
                }
                Subject<T> subject = publisher;
                if (subject != null)
                {
                    try
                    {
                        subject.OnCompleted();
                    }
                    finally
                    {
                        subject.Dispose();
                        publisher = null;
                    }
                }
            }
        }

        public override string ToString()
        {
            return value != null ? value.ToString() : "(null)";
        }

        public bool IsRequiredSubscribeOnCurrentThread()
        {
            return false;
        }
    }
}
