using System;
using UnityEngine;

namespace UnityStandardAssets.CinematicEffects
{
    [ExecuteInEditMode]
    [RequireComponent(typeof(Camera))]
    [AddComponentMenu("Image Effects/Cinematic/Tonemapping and Color Grading")]
    [ImageEffectAllowedInSceneView]
    public class TonemappingColorGrading : MonoBehaviour
    {
        [AttributeUsage(AttributeTargets.Field)]
        public class SettingsGroup : Attribute
        {
        }

        public class IndentedGroup : PropertyAttribute
        {
        }

        public class ChannelMixer : PropertyAttribute
        {
        }

        public class ColorWheelGroup : PropertyAttribute
        {
            public int minSizePerWheel = 60;

            public int maxSizePerWheel = 150;

            public ColorWheelGroup()
            {
            }

            public ColorWheelGroup(int minSizePerWheel, int maxSizePerWheel)
            {
                this.minSizePerWheel = minSizePerWheel;
                this.maxSizePerWheel = maxSizePerWheel;
            }
        }

        public class Curve : PropertyAttribute
        {
            public Color color = Color.white;

            public Curve()
            {
            }

            public Curve(float r, float g, float b, float a)
            {
                color = new Color(r, g, b, a);
            }
        }

        [Serializable]
        public struct EyeAdaptationSettings
        {
            public bool enabled;

            [Min(0f)]
            [Tooltip("Midpoint Adjustment.")]
            public float middleGrey;

            [Tooltip("The lowest possible exposure value; adjust this value to modify the brightest areas of your level.")]
            public float min;

            [Tooltip("The highest possible exposure value; adjust this value to modify the darkest areas of your level.")]
            public float max;

            [Min(0f)]
            [Tooltip("Speed of linear adaptation. Higher is faster.")]
            public float speed;

            [Tooltip("Displays a luminosity helper in the GameView.")]
            public bool showDebug;

            public static EyeAdaptationSettings defaultSettings
            {
                get
                {
                    EyeAdaptationSettings result = default(EyeAdaptationSettings);
                    result.enabled = false;
                    result.showDebug = false;
                    result.middleGrey = 0.5f;
                    result.min = -3f;
                    result.max = 3f;
                    result.speed = 1.5f;
                    return result;
                }
            }
        }

        public enum Tonemapper
        {
            ACES,
            Curve,
            Hable,
            HejlDawson,
            Photographic,
            Reinhard,
            Neutral
        }

        [Serializable]
        public struct TonemappingSettings
        {
            public bool enabled;

            [Tooltip("Tonemapping technique to use. ACES is the recommended one.")]
            public Tonemapper tonemapper;

            [Min(0f)]
            [Tooltip("Adjusts the overall exposure of the scene.")]
            public float exposure;

            [Tooltip("Custom tonemapping curve.")]
            public AnimationCurve curve;

            [Range(-0.1f, 0.1f)]
            public float neutralBlackIn;

            [Range(1f, 20f)]
            public float neutralWhiteIn;

            [Range(-0.09f, 0.1f)]
            public float neutralBlackOut;

            [Range(1f, 19f)]
            public float neutralWhiteOut;

            [Range(0.1f, 20f)]
            public float neutralWhiteLevel;

            [Range(1f, 10f)]
            public float neutralWhiteClip;

            public static TonemappingSettings defaultSettings
            {
                get
                {
                    TonemappingSettings result = default(TonemappingSettings);
                    result.enabled = false;
                    result.tonemapper = Tonemapper.Neutral;
                    result.exposure = 1f;
                    result.curve = CurvesSettings.defaultCurve;
                    result.neutralBlackIn = 0.02f;
                    result.neutralWhiteIn = 10f;
                    result.neutralBlackOut = 0f;
                    result.neutralWhiteOut = 10f;
                    result.neutralWhiteLevel = 5.3f;
                    result.neutralWhiteClip = 10f;
                    return result;
                }
            }
        }

        [Serializable]
        public struct LUTSettings
        {
            public bool enabled;

            [Tooltip("Custom lookup texture (strip format, e.g. 256x16).")]
            public Texture texture;

            [Range(0f, 1f)]
            [Tooltip("Blending factor.")]
            public float contribution;

            public static LUTSettings defaultSettings
            {
                get
                {
                    LUTSettings result = default(LUTSettings);
                    result.enabled = false;
                    result.texture = null;
                    result.contribution = 1f;
                    return result;
                }
            }
        }

        [Serializable]
        public struct ColorWheelsSettings
        {
            [ColorUsage(false)]
            public Color shadows;

            [ColorUsage(false)]
            public Color midtones;

            [ColorUsage(false)]
            public Color highlights;

            public static ColorWheelsSettings defaultSettings
            {
                get
                {
                    ColorWheelsSettings result = default(ColorWheelsSettings);
                    result.shadows = Color.white;
                    result.midtones = Color.white;
                    result.highlights = Color.white;
                    return result;
                }
            }
        }

        [Serializable]
        public struct BasicsSettings
        {
            [Range(-2f, 2f)]
            [Tooltip("Sets the white balance to a custom color temperature.")]
            public float temperatureShift;

            [Range(-2f, 2f)]
            [Tooltip("Sets the white balance to compensate for a green or magenta tint.")]
            public float tint;

            [Space]
            [Range(-0.5f, 0.5f)]
            [Tooltip("Shift the hue of all colors.")]
            public float hue;

            [Range(0f, 2f)]
            [Tooltip("Pushes the intensity of all colors.")]
            public float saturation;

            [Range(-1f, 1f)]
            [Tooltip("Adjusts the saturation so that clipping is minimized as colors approach full saturation.")]
            public float vibrance;

            [Range(0f, 10f)]
            [Tooltip("Brightens or darkens all colors.")]
            public float value;

            [Space]
            [Range(0f, 2f)]
            [Tooltip("Expands or shrinks the overall range of tonal values.")]
            public float contrast;

            [Range(0.01f, 5f)]
            [Tooltip("Contrast gain curve. Controls the steepness of the curve.")]
            public float gain;

            [Range(0.01f, 5f)]
            [Tooltip("Applies a pow function to the source.")]
            public float gamma;

            public static BasicsSettings defaultSettings
            {
                get
                {
                    BasicsSettings result = default(BasicsSettings);
                    result.temperatureShift = 0f;
                    result.tint = 0f;
                    result.contrast = 1f;
                    result.hue = 0f;
                    result.saturation = 1f;
                    result.value = 1f;
                    result.vibrance = 0f;
                    result.gain = 1f;
                    result.gamma = 1f;
                    return result;
                }
            }
        }

        [Serializable]
        public struct ChannelMixerSettings
        {
            public int currentChannel;

            public Vector3[] channels;

            public static ChannelMixerSettings defaultSettings
            {
                get
                {
                    ChannelMixerSettings result = default(ChannelMixerSettings);
                    result.currentChannel = 0;
                    result.channels = new Vector3[3]
                    {
                        new Vector3(1f, 0f, 0f),
                        new Vector3(0f, 1f, 0f),
                        new Vector3(0f, 0f, 1f)
                    };
                    return result;
                }
            }
        }

        [Serializable]
        public struct CurvesSettings
        {
            [Curve]
            public AnimationCurve master;

            [Curve(1f, 0f, 0f, 1f)]
            public AnimationCurve red;

            [Curve(0f, 1f, 0f, 1f)]
            public AnimationCurve green;

            [Curve(0f, 1f, 1f, 1f)]
            public AnimationCurve blue;

            public static CurvesSettings defaultSettings
            {
                get
                {
                    CurvesSettings result = default(CurvesSettings);
                    result.master = defaultCurve;
                    result.red = defaultCurve;
                    result.green = defaultCurve;
                    result.blue = defaultCurve;
                    return result;
                }
            }

            public static AnimationCurve defaultCurve
            {
                get
                {
                    return new AnimationCurve(new Keyframe(0f, 0f, 1f, 1f), new Keyframe(1f, 1f, 1f, 1f));
                }
            }
        }

        public enum ColorGradingPrecision
        {
            Normal = 0x10,
            High = 0x20
        }

        [Serializable]
        public struct ColorGradingSettings
        {
            public bool enabled;

            [Tooltip("Internal LUT precision. \"Normal\" is 256x16, \"High\" is 1024x32. Prefer \"Normal\" on mobile devices.")]
            public ColorGradingPrecision precision;

            [Space]
            [ColorWheelGroup]
            public ColorWheelsSettings colorWheels;

            [Space]
            [IndentedGroup]
            public BasicsSettings basics;

            [Space]
            [ChannelMixer]
            public ChannelMixerSettings channelMixer;

            [Space]
            [IndentedGroup]
            public CurvesSettings curves;

            [Space]
            [Tooltip("Use dithering to try and minimize color banding in dark areas.")]
            public bool useDithering;

            [Tooltip("Displays the generated LUT in the top left corner of the GameView.")]
            public bool showDebug;

            public static ColorGradingSettings defaultSettings
            {
                get
                {
                    ColorGradingSettings result = default(ColorGradingSettings);
                    result.enabled = false;
                    result.useDithering = false;
                    result.showDebug = false;
                    result.precision = ColorGradingPrecision.Normal;
                    result.colorWheels = ColorWheelsSettings.defaultSettings;
                    result.basics = BasicsSettings.defaultSettings;
                    result.channelMixer = ChannelMixerSettings.defaultSettings;
                    result.curves = CurvesSettings.defaultSettings;
                    return result;
                }
            }

            internal void Reset()
            {
                curves = CurvesSettings.defaultSettings;
            }
        }

        private enum Pass
        {
            LutGen,
            AdaptationLog,
            AdaptationExpBlend,
            AdaptationExp,
            TonemappingOff,
            TonemappingACES,
            TonemappingCurve,
            TonemappingHable,
            TonemappingHejlDawson,
            TonemappingPhotographic,
            TonemappingReinhard,
            TonemappingNeutral,
            AdaptationDebug
        }

        [SerializeField]
        [SettingsGroup]
        private EyeAdaptationSettings m_EyeAdaptation = EyeAdaptationSettings.defaultSettings;

        [SerializeField]
        [SettingsGroup]
        private TonemappingSettings m_Tonemapping = TonemappingSettings.defaultSettings;

        [SerializeField]
        [SettingsGroup]
        private ColorGradingSettings m_ColorGrading = ColorGradingSettings.defaultSettings;

        [SerializeField]
        [SettingsGroup]
        private LUTSettings m_Lut = LUTSettings.defaultSettings;

        private Texture2D m_IdentityLut;

        private RenderTexture m_InternalLut;

        private Texture2D m_CurveTexture;

        private Texture2D m_TonemapperCurve;

        private float m_TonemapperCurveRange;

        [SerializeField]
        private Shader m_Shader;

        private Material m_Material;

        private bool m_Dirty = true;

        private bool m_TonemapperDirty = true;

        private RenderTexture m_SmallAdaptiveRt;

        private RenderTextureFormat m_AdaptiveRtFormat;

        public EyeAdaptationSettings eyeAdaptation
        {
            get
            {
                return m_EyeAdaptation;
            }
            set
            {
                m_EyeAdaptation = value;
            }
        }

        public TonemappingSettings tonemapping
        {
            get
            {
                return m_Tonemapping;
            }
            set
            {
                m_Tonemapping = value;
                SetTonemapperDirty();
            }
        }

        public ColorGradingSettings colorGrading
        {
            get
            {
                return m_ColorGrading;
            }
            set
            {
                m_ColorGrading = value;
                SetDirty();
            }
        }

        public LUTSettings lut
        {
            get
            {
                return m_Lut;
            }
            set
            {
                m_Lut = value;
            }
        }

        private Texture2D identityLut
        {
            get
            {
                if (m_IdentityLut == null || m_IdentityLut.height != lutSize)
                {
                    DestroyImmediate(m_IdentityLut);
                    m_IdentityLut = GenerateIdentityLut(lutSize);
                }
                return m_IdentityLut;
            }
        }

        private RenderTexture internalLutRt
        {
            get
            {
                if (m_InternalLut == null || !m_InternalLut.IsCreated() || m_InternalLut.height != lutSize)
                {
                    DestroyImmediate(m_InternalLut);
                    m_InternalLut = new RenderTexture(lutSize * lutSize, lutSize, 0, RenderTextureFormat.ARGB32)
                    {
                        name = "Internal LUT",
                        filterMode = FilterMode.Bilinear,
                        anisoLevel = 0,
                        hideFlags = HideFlags.DontSave
                    };
                }
                return m_InternalLut;
            }
        }

        private Texture2D curveTexture
        {
            get
            {
                if (m_CurveTexture == null)
                {
                    m_CurveTexture = new Texture2D(256, 1, TextureFormat.ARGB32, false, true)
                    {
                        name = "Curve texture",
                        wrapMode = TextureWrapMode.Clamp,
                        filterMode = FilterMode.Bilinear,
                        anisoLevel = 0,
                        hideFlags = HideFlags.DontSave
                    };
                }
                return m_CurveTexture;
            }
        }

        private Texture2D tonemapperCurve
        {
            get
            {
                if (m_TonemapperCurve == null)
                {
                    TextureFormat format = TextureFormat.RGB24;
                    if (SystemInfo.SupportsTextureFormat(TextureFormat.RFloat))
                    {
                        format = TextureFormat.RFloat;
                    }
                    else if (SystemInfo.SupportsTextureFormat(TextureFormat.RHalf))
                    {
                        format = TextureFormat.RHalf;
                    }
                    m_TonemapperCurve = new Texture2D(256, 1, format, false, true)
                    {
                        name = "Tonemapper curve texture",
                        wrapMode = TextureWrapMode.Clamp,
                        filterMode = FilterMode.Bilinear,
                        anisoLevel = 0,
                        hideFlags = HideFlags.DontSave
                    };
                }
                return m_TonemapperCurve;
            }
        }

        public Shader shader
        {
            get
            {
                if (m_Shader == null)
                {
                    m_Shader = Shader.Find("Hidden/TonemappingColorGrading");
                }
                return m_Shader;
            }
        }

        public Material material
        {
            get
            {
                if (m_Material == null)
                {
                    m_Material = ImageEffectHelper.CheckShaderAndCreateMaterial(shader);
                }
                return m_Material;
            }
        }

        public bool isGammaColorSpace
        {
            get
            {
                return QualitySettings.activeColorSpace == ColorSpace.Gamma;
            }
        }

        public int lutSize
        {
            get
            {
                ColorGradingSettings colorGrading = this.colorGrading;
                return (int)colorGrading.precision;
            }
        }

        public bool validRenderTextureFormat
        {
            get;
            private set;
        }

        public bool validUserLutSize
        {
            get;
            private set;
        }

        public void SetDirty()
        {
            m_Dirty = true;
        }

        public void SetTonemapperDirty()
        {
            m_TonemapperDirty = true;
        }

        private void OnEnable()
        {
            if (!ImageEffectHelper.IsSupported(shader, false, true, this))
            {
                enabled = false;
                return;
            }
            SetDirty();
            SetTonemapperDirty();
        }

        private void OnDisable()
        {
            if (m_Material != null)
            {
                DestroyImmediate(m_Material);
            }
            if (m_IdentityLut != null)
            {
                DestroyImmediate(m_IdentityLut);
            }
            if (m_InternalLut != null)
            {
                DestroyImmediate(internalLutRt);
            }
            if (m_SmallAdaptiveRt != null)
            {
                DestroyImmediate(m_SmallAdaptiveRt);
            }
            if (m_CurveTexture != null)
            {
                DestroyImmediate(m_CurveTexture);
            }
            if (m_TonemapperCurve != null)
            {
                DestroyImmediate(m_TonemapperCurve);
            }
            m_Material = null;
            m_IdentityLut = null;
            m_InternalLut = null;
            m_SmallAdaptiveRt = null;
            m_CurveTexture = null;
            m_TonemapperCurve = null;
        }

        private void OnValidate()
        {
            SetDirty();
            SetTonemapperDirty();
        }

        private static Texture2D GenerateIdentityLut(int dim)
        {
            Color[] array = new Color[dim * dim * dim];
            float num = 1f / (dim - 1f);
            for (int i = 0; i < dim; i++)
            {
                for (int j = 0; j < dim; j++)
                {
                    for (int k = 0; k < dim; k++)
                    {
                        array[i + j * dim + k * dim * dim] = new Color(i * num, Mathf.Abs(k * num), j * num, 1f);
                    }
                }
            }
            Texture2D texture2D = new Texture2D(dim * dim, dim, TextureFormat.RGB24, false, true);
            texture2D.name = "Identity LUT";
            texture2D.filterMode = FilterMode.Bilinear;
            texture2D.anisoLevel = 0;
            texture2D.hideFlags = HideFlags.DontSave;
            Texture2D texture2D2 = texture2D;
            texture2D2.SetPixels(array);
            texture2D2.Apply();
            return texture2D2;
        }

        private float StandardIlluminantY(float x)
        {
            return 2.87f * x - 3f * x * x - 0.275095075f;
        }

        private Vector3 CIExyToLMS(float x, float y)
        {
            float num = 1f;
            float num2 = num * x / y;
            float num3 = num * (1f - x - y) / y;
            float x2 = 0.7328f * num2 + 0.4296f * num - 0.1624f * num3;
            float y2 = -0.7036f * num2 + 1.6975f * num + 0.0061f * num3;
            float z = 0.003f * num2 + 0.0136f * num + 0.9834f * num3;
            return new Vector3(x2, y2, z);
        }

        private Vector3 GetWhiteBalance()
        {
            ColorGradingSettings colorGrading = this.colorGrading;
            float temperatureShift = colorGrading.basics.temperatureShift;
            ColorGradingSettings colorGrading2 = this.colorGrading;
            float tint = colorGrading2.basics.tint;
            float x = 0.31271f - temperatureShift * (!(temperatureShift < 0f) ? 0.05f : 0.1f);
            float y = StandardIlluminantY(x) + tint * 0.05f;
            Vector3 vector = new Vector3(0.949237f, 1.03542f, 1.08728f);
            Vector3 vector2 = CIExyToLMS(x, y);
            return new Vector3(vector.x / vector2.x, vector.y / vector2.y, vector.z / vector2.z);
        }

        private static Color NormalizeColor(Color c)
        {
            float num = (c.r + c.g + c.b) / 3f;
            if (Mathf.Approximately(num, 0f))
            {
                return new Color(1f, 1f, 1f, 1f);
            }
            Color result = default(Color);
            result.r = c.r / num;
            result.g = c.g / num;
            result.b = c.b / num;
            result.a = 1f;
            return result;
        }

        private void GenerateLiftGammaGain(out Color lift, out Color gamma, out Color gain)
        {
            ColorGradingSettings colorGrading = this.colorGrading;
            Color color = NormalizeColor(colorGrading.colorWheels.shadows);
            ColorGradingSettings colorGrading2 = this.colorGrading;
            Color color2 = NormalizeColor(colorGrading2.colorWheels.midtones);
            ColorGradingSettings colorGrading3 = this.colorGrading;
            Color color3 = NormalizeColor(colorGrading3.colorWheels.highlights);
            float num = (color.r + color.g + color.b) / 3f;
            float num2 = (color2.r + color2.g + color2.b) / 3f;
            float num3 = (color3.r + color3.g + color3.b) / 3f;
            float r = (color.r - num) * 0.1f;
            float g = (color.g - num) * 0.1f;
            float b = (color.b - num) * 0.1f;
            float b2 = Mathf.Pow(2f, (color2.r - num2) * 0.5f);
            float b3 = Mathf.Pow(2f, (color2.g - num2) * 0.5f);
            float b4 = Mathf.Pow(2f, (color2.b - num2) * 0.5f);
            float r2 = Mathf.Pow(2f, (color3.r - num3) * 0.5f);
            float g2 = Mathf.Pow(2f, (color3.g - num3) * 0.5f);
            float b5 = Mathf.Pow(2f, (color3.b - num3) * 0.5f);
            float r3 = 1f / Mathf.Max(0.01f, b2);
            float g3 = 1f / Mathf.Max(0.01f, b3);
            float b6 = 1f / Mathf.Max(0.01f, b4);
            lift = new Color(r, g, b);
            gamma = new Color(r3, g3, b6);
            gain = new Color(r2, g2, b5);
        }

        private void GenCurveTexture()
        {
            ColorGradingSettings colorGrading = this.colorGrading;
            AnimationCurve master = colorGrading.curves.master;
            ColorGradingSettings colorGrading2 = this.colorGrading;
            AnimationCurve red = colorGrading2.curves.red;
            ColorGradingSettings colorGrading3 = this.colorGrading;
            AnimationCurve green = colorGrading3.curves.green;
            ColorGradingSettings colorGrading4 = this.colorGrading;
            AnimationCurve blue = colorGrading4.curves.blue;
            Color[] array = new Color[256];
            for (float num = 0f; num <= 1f; num += 0.003921569f)
            {
                float a = Mathf.Clamp(master.Evaluate(num), 0f, 1f);
                float r = Mathf.Clamp(red.Evaluate(num), 0f, 1f);
                float g = Mathf.Clamp(green.Evaluate(num), 0f, 1f);
                float b = Mathf.Clamp(blue.Evaluate(num), 0f, 1f);
                array[(int)Mathf.Floor(num * 255f)] = new Color(r, g, b, a);
            }
            curveTexture.SetPixels(array);
            curveTexture.Apply();
        }

        private bool CheckUserLut()
        {
            LUTSettings lut = this.lut;
            int height = lut.texture.height;
            LUTSettings lut2 = this.lut;
            validUserLutSize = height == (int)Mathf.Sqrt(lut2.texture.width);
            return validUserLutSize;
        }

        private bool CheckSmallAdaptiveRt()
        {
            if (m_SmallAdaptiveRt != null)
            {
                return false;
            }
            m_AdaptiveRtFormat = RenderTextureFormat.ARGBHalf;
            if (SystemInfo.SupportsRenderTextureFormat(RenderTextureFormat.RGHalf))
            {
                m_AdaptiveRtFormat = RenderTextureFormat.RGHalf;
            }
            m_SmallAdaptiveRt = new RenderTexture(1, 1, 0, m_AdaptiveRtFormat);
            m_SmallAdaptiveRt.hideFlags = HideFlags.DontSave;
            return true;
        }

        private void OnGUI()
        {
            if (Event.current.type != EventType.Repaint)
            {
                return;
            }
            int num = 0;
            if (m_InternalLut != null)
            {
                ColorGradingSettings colorGrading = this.colorGrading;
                if (colorGrading.enabled)
                {
                    ColorGradingSettings colorGrading2 = this.colorGrading;
                    if (colorGrading2.showDebug)
                    {
                        Graphics.DrawTexture(new Rect(0f, num, lutSize * lutSize, lutSize), internalLutRt);
                        num += lutSize;
                    }
                }
            }
            if (!(m_SmallAdaptiveRt != null))
            {
                return;
            }
            EyeAdaptationSettings eyeAdaptation = this.eyeAdaptation;
            if (eyeAdaptation.enabled)
            {
                EyeAdaptationSettings eyeAdaptation2 = this.eyeAdaptation;
                if (eyeAdaptation2.showDebug)
                {
                    m_Material.SetPass(12);
                    Graphics.DrawTexture(new Rect(0f, num, 256f, 16f), m_SmallAdaptiveRt, m_Material);
                }
            }
        }

        [ImageEffectTransformsToLDR]
        private void OnRenderImage(RenderTexture source, RenderTexture destination)
        {
            material.shaderKeywords = null;
            RenderTexture renderTexture = null;
            RenderTexture[] array = null;
            EyeAdaptationSettings eyeAdaptation = this.eyeAdaptation;
            if (eyeAdaptation.enabled)
            {
                bool flag = CheckSmallAdaptiveRt();
                int num = source.width >= source.height ? source.height : source.width;
                int num2 = num;
                num2 |= num2 >> 1;
                num2 |= num2 >> 2;
                num2 |= num2 >> 4;
                num2 |= num2 >> 8;
                num2 |= num2 >> 16;
                num2 -= num2 >> 1;
                renderTexture = RenderTexture.GetTemporary(num2, num2, 0, m_AdaptiveRtFormat);
                Graphics.Blit(source, renderTexture);
                int num3 = (int)Mathf.Log(renderTexture.width, 2f);
                int num4 = 2;
                array = new RenderTexture[num3];
                for (int i = 0; i < num3; i++)
                {
                    array[i] = RenderTexture.GetTemporary(renderTexture.width / num4, renderTexture.width / num4, 0, m_AdaptiveRtFormat);
                    num4 <<= 1;
                }
                RenderTexture source2 = array[num3 - 1];
                Graphics.Blit(renderTexture, array[0], this.material, 1);
                for (int j = 0; j < num3 - 1; j++)
                {
                    Graphics.Blit(array[j], array[j + 1]);
                    source2 = array[j + 1];
                }
                m_SmallAdaptiveRt.MarkRestoreExpected();
                Material material = this.material;
                EyeAdaptationSettings eyeAdaptation2 = this.eyeAdaptation;
                material.SetFloat("_AdaptationSpeed", Mathf.Max(eyeAdaptation2.speed, 0.001f));
                Graphics.Blit(source2, m_SmallAdaptiveRt, this.material, !flag ? 2 : 3);
                Material material2 = this.material;
                EyeAdaptationSettings eyeAdaptation3 = this.eyeAdaptation;
                material2.SetFloat("_MiddleGrey", eyeAdaptation3.middleGrey);
                Material material3 = this.material;
                EyeAdaptationSettings eyeAdaptation4 = this.eyeAdaptation;
                material3.SetFloat("_AdaptationMin", Mathf.Pow(2f, eyeAdaptation4.min));
                Material material4 = this.material;
                EyeAdaptationSettings eyeAdaptation5 = this.eyeAdaptation;
                material4.SetFloat("_AdaptationMax", Mathf.Pow(2f, eyeAdaptation5.max));
                this.material.SetTexture("_LumTex", m_SmallAdaptiveRt);
                this.material.EnableKeyword("ENABLE_EYE_ADAPTATION");
            }
            int num5 = 4;
            TonemappingSettings tonemapping = this.tonemapping;
            if (tonemapping.enabled)
            {
                TonemappingSettings tonemapping2 = this.tonemapping;
                if (tonemapping2.tonemapper == Tonemapper.Curve)
                {
                    if (m_TonemapperDirty)
                    {
                        float num6 = 1f;
                        TonemappingSettings tonemapping3 = this.tonemapping;
                        if (tonemapping3.curve.length > 0)
                        {
                            TonemappingSettings tonemapping4 = this.tonemapping;
                            AnimationCurve curve = tonemapping4.curve;
                            TonemappingSettings tonemapping5 = this.tonemapping;
                            num6 = curve[tonemapping5.curve.length - 1].time;
                            for (float num7 = 0f; num7 <= 1f; num7 += 0.003921569f)
                            {
                                TonemappingSettings tonemapping6 = this.tonemapping;
                                float num8 = tonemapping6.curve.Evaluate(num7 * num6);
                                tonemapperCurve.SetPixel(Mathf.FloorToInt(num7 * 255f), 0, new Color(num8, num8, num8));
                            }
                            tonemapperCurve.Apply();
                        }
                        m_TonemapperCurveRange = 1f / num6;
                        m_TonemapperDirty = false;
                    }
                    material.SetFloat("_ToneCurveRange", m_TonemapperCurveRange);
                    material.SetTexture("_ToneCurve", tonemapperCurve);
                }
                else
                {
                    TonemappingSettings tonemapping7 = this.tonemapping;
                    if (tonemapping7.tonemapper == Tonemapper.Neutral)
                    {
                        TonemappingSettings tonemapping8 = this.tonemapping;
                        float num9 = tonemapping8.neutralBlackIn * 20f + 1f;
                        TonemappingSettings tonemapping9 = this.tonemapping;
                        float num10 = tonemapping9.neutralBlackOut * 10f + 1f;
                        TonemappingSettings tonemapping10 = this.tonemapping;
                        float num11 = tonemapping10.neutralWhiteIn / 20f;
                        TonemappingSettings tonemapping11 = this.tonemapping;
                        float num12 = 1f - tonemapping11.neutralWhiteOut / 20f;
                        float t = num9 / num10;
                        float t2 = num11 / num12;
                        float y = Mathf.Max(0f, Mathf.LerpUnclamped(0.57f, 0.37f, t));
                        float z = Mathf.LerpUnclamped(0.01f, 0.24f, t2);
                        float w = Mathf.Max(0f, Mathf.LerpUnclamped(0.02f, 0.2f, t));
                        material.SetVector("_NeutralTonemapperParams1", new Vector4(0.2f, y, z, w));
                        Material material5 = material;
                        TonemappingSettings tonemapping12 = this.tonemapping;
                        float neutralWhiteLevel = tonemapping12.neutralWhiteLevel;
                        TonemappingSettings tonemapping13 = this.tonemapping;
                        material5.SetVector("_NeutralTonemapperParams2", new Vector4(0.02f, 0.3f, neutralWhiteLevel, tonemapping13.neutralWhiteClip / 10f));
                    }
                }
                Material material6 = material;
                TonemappingSettings tonemapping14 = this.tonemapping;
                material6.SetFloat("_Exposure", tonemapping14.exposure);
                int num13 = num5;
                TonemappingSettings tonemapping15 = this.tonemapping;
                num5 = (int)(num13 + (tonemapping15.tonemapper + 1));
            }
            ColorGradingSettings colorGrading = this.colorGrading;
            if (colorGrading.enabled)
            {
                if (m_Dirty || !m_InternalLut.IsCreated())
                {
                    Color lift;
                    Color gamma;
                    Color gain;
                    GenerateLiftGammaGain(out lift, out gamma, out gain);
                    GenCurveTexture();
                    material.SetVector("_WhiteBalance", GetWhiteBalance());
                    material.SetVector("_Lift", lift);
                    material.SetVector("_Gamma", gamma);
                    material.SetVector("_Gain", gain);
                    Material material7 = material;
                    ColorGradingSettings colorGrading2 = this.colorGrading;
                    float contrast = colorGrading2.basics.contrast;
                    ColorGradingSettings colorGrading3 = this.colorGrading;
                    float gain2 = colorGrading3.basics.gain;
                    ColorGradingSettings colorGrading4 = this.colorGrading;
                    material7.SetVector("_ContrastGainGamma", new Vector3(contrast, gain2, 1f / colorGrading4.basics.gamma));
                    Material material8 = material;
                    ColorGradingSettings colorGrading5 = this.colorGrading;
                    material8.SetFloat("_Vibrance", colorGrading5.basics.vibrance);
                    Material material9 = material;
                    ColorGradingSettings colorGrading6 = this.colorGrading;
                    float hue = colorGrading6.basics.hue;
                    ColorGradingSettings colorGrading7 = this.colorGrading;
                    float saturation = colorGrading7.basics.saturation;
                    ColorGradingSettings colorGrading8 = this.colorGrading;
                    material9.SetVector("_HSV", new Vector4(hue, saturation, colorGrading8.basics.value));
                    Material material10 = material;
                    ColorGradingSettings colorGrading9 = this.colorGrading;
                    material10.SetVector("_ChannelMixerRed", colorGrading9.channelMixer.channels[0]);
                    Material material11 = material;
                    ColorGradingSettings colorGrading10 = this.colorGrading;
                    material11.SetVector("_ChannelMixerGreen", colorGrading10.channelMixer.channels[1]);
                    Material material12 = material;
                    ColorGradingSettings colorGrading11 = this.colorGrading;
                    material12.SetVector("_ChannelMixerBlue", colorGrading11.channelMixer.channels[2]);
                    material.SetTexture("_CurveTex", curveTexture);
                    internalLutRt.MarkRestoreExpected();
                    Graphics.Blit(identityLut, internalLutRt, material, 0);
                    m_Dirty = false;
                }
                material.EnableKeyword("ENABLE_COLOR_GRADING");
                ColorGradingSettings colorGrading12 = this.colorGrading;
                if (colorGrading12.useDithering)
                {
                    material.EnableKeyword("ENABLE_DITHERING");
                }
                material.SetTexture("_InternalLutTex", internalLutRt);
                material.SetVector("_InternalLutParams", new Vector3(1f / internalLutRt.width, 1f / internalLutRt.height, internalLutRt.height - 1f));
            }
            LUTSettings lut = this.lut;
            if (lut.enabled)
            {
                LUTSettings lut2 = this.lut;
                if (lut2.texture != null && CheckUserLut())
                {
                    Material material13 = material;
                    LUTSettings lut3 = this.lut;
                    material13.SetTexture("_UserLutTex", lut3.texture);
                    Material material14 = material;
                    LUTSettings lut4 = this.lut;
                    float x = 1f / lut4.texture.width;
                    LUTSettings lut5 = this.lut;
                    float y2 = 1f / lut5.texture.height;
                    LUTSettings lut6 = this.lut;
                    float z2 = lut6.texture.height - 1f;
                    LUTSettings lut7 = this.lut;
                    material14.SetVector("_UserLutParams", new Vector4(x, y2, z2, lut7.contribution));
                    material.EnableKeyword("ENABLE_USER_LUT");
                }
            }
            Graphics.Blit(source, destination, material, num5);
            EyeAdaptationSettings eyeAdaptation6 = this.eyeAdaptation;
            if (eyeAdaptation6.enabled)
            {
                for (int k = 0; k < array.Length; k++)
                {
                    RenderTexture.ReleaseTemporary(array[k]);
                }
                RenderTexture.ReleaseTemporary(renderTexture);
            }
        }

        public Texture2D BakeLUT()
        {
            Texture2D texture2D = new Texture2D(internalLutRt.width, internalLutRt.height, TextureFormat.RGB24, false, true);
            RenderTexture.active = internalLutRt;
            texture2D.ReadPixels(new Rect(0f, 0f, texture2D.width, texture2D.height), 0, 0);
            RenderTexture.active = null;
            return texture2D;
        }
    }
}
