using UnityEngine;

namespace UnityStandardAssets.ImageEffects
{
    [ExecuteInEditMode]
    [AddComponentMenu("Image Effects/Color Adjustments/Color Correction (3D Lookup Texture)")]
    public class ColorCorrectionLookup : PostEffectsBase
    {
        public Shader shader;

        private Material material;

        public Texture3D converted3DLut;

        public string basedOnTempTex = string.Empty;

        public override bool CheckResources()
        {
            CheckSupport(false);
            material = CheckShaderAndCreateMaterial(shader, material);
            if (!isSupported || !SystemInfo.supports3DTextures)
            {
                ReportAutoDisable();
            }
            return isSupported;
        }

        private void OnDisable()
        {
            if ((bool)material)
            {
                DestroyImmediate(material);
                material = null;
            }
        }

        private void OnDestroy()
        {
            if ((bool)converted3DLut)
            {
                DestroyImmediate(converted3DLut);
            }
            converted3DLut = null;
        }

        public void SetIdentityLut()
        {
            int num = 16;
            Color[] array = new Color[num * num * num];
            float num2 = 1f / (1f * num - 1f);
            for (int i = 0; i < num; i++)
            {
                for (int j = 0; j < num; j++)
                {
                    for (int k = 0; k < num; k++)
                    {
                        array[i + j * num + k * num * num] = new Color(i * 1f * num2, j * 1f * num2, k * 1f * num2, 1f);
                    }
                }
            }
            if ((bool)converted3DLut)
            {
                DestroyImmediate(converted3DLut);
            }
            converted3DLut = new Texture3D(num, num, num, TextureFormat.ARGB32, false);
            converted3DLut.SetPixels(array);
            converted3DLut.Apply();
            basedOnTempTex = string.Empty;
        }

        public bool ValidDimensions(Texture2D tex2d)
        {
            if (!tex2d)
            {
                return false;
            }
            int height = tex2d.height;
            if (height != Mathf.FloorToInt(Mathf.Sqrt(tex2d.width)))
            {
                return false;
            }
            return true;
        }

        public void Convert(Texture2D temp2DTex, string path)
        {
            if ((bool)temp2DTex)
            {
                var num = temp2DTex.height;
                if (!ValidDimensions(temp2DTex))
                {
                    Debug.LogWarning("The given 2D texture " + temp2DTex.name + " cannot be used as a 3D LUT.");
                    basedOnTempTex = string.Empty;
                    return;
                }
                Color[] pixels = temp2DTex.GetPixels();
                Color[] array = new Color[pixels.Length];
                for (int i = 0; i < num; i++)
                {
                    for (int j = 0; j < num; j++)
                    {
                        for (int k = 0; k < num; k++)
                        {
                            int num2 = num - j - 1;
                            array[i + j * num + k * num * num] = pixels[k * num + i + num2 * num * num];
                        }
                    }
                }
                if ((bool)converted3DLut)
                {
                    DestroyImmediate(converted3DLut);
                }
                converted3DLut = new Texture3D(num, num, num, TextureFormat.ARGB32, false);
                converted3DLut.SetPixels(array);
                converted3DLut.Apply();
                basedOnTempTex = path;
            }
            else
            {
                Debug.LogError("Couldn't color correct with 3D LUT texture. Image Effect will be disabled.");
            }
        }

        private void OnRenderImage(RenderTexture source, RenderTexture destination)
        {
            if (!CheckResources() || !SystemInfo.supports3DTextures)
            {
                Graphics.Blit(source, destination);
                return;
            }
            if (converted3DLut == null)
            {
                SetIdentityLut();
            }
            int width = converted3DLut.width;
            converted3DLut.wrapMode = TextureWrapMode.Clamp;
            material.SetFloat("_Scale", (width - 1) / (1f * width));
            material.SetFloat("_Offset", 1f / (2f * width));
            material.SetTexture("_ClutTex", converted3DLut);
            Graphics.Blit(source, destination, material, QualitySettings.activeColorSpace == ColorSpace.Linear ? 1 : 0);
        }
    }
}
